import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/publisher_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/promotion_get_filter_query.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/promotion_get_promotion_detail_query.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/promotion_get_promotions_query.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';

import 'package:dio/dio.dart';

abstract class PromotionRemoteDataSource {
  Future<Either<Failure, dynamic>> getPromotions({
    int? start,
    int? limit,
    List<String>? categoryIds,
    List<String>? storeIds,
    List<String>? retailerTokens,
    List<String>? companyTokens,
    String? textSearch,
    String? zipcode,
    String? latitude,
    String? longitude,
    String? radius,
    String? orderBy,
    String? publisherToken,
  });

  Future<Either<Failure, dynamic>> getPromotionDetail({
    String? promotionToken,
    String? publisherToken,
  });

  Future<Either<Failure, dynamic>> getCategories({
    String? publisherToken,
  });

  Future<Either<Failure, dynamic>> getRetailers({
    String? publisherToken,
    int? start,
    int? limit,
  });

  Future<Either<Failure, dynamic>> getCompanies({
    String? publisherToken,
    int? start,
    int? limit,
  });
}

class PromotionRemoteDataSourceImpl implements PromotionRemoteDataSource {
  Dio dio = Dio()..interceptors.addAll(ConfigReader.dioInterceptors);

  @override
  Future<Either<Failure, dynamic>> getPromotions({
    int? start,
    int? limit,
    List<String>? categoryIds,
    List<String>? storeIds,
    List<String>? retailerTokens,
    List<String>? companyTokens,
    String? textSearch,
    String? zipcode,
    String? latitude,
    String? longitude,
    String? radius,
    String? orderBy,
    String? publisherToken,
  }) async {
    dio.options.headers = PublisherApiHelper.buildHeaders();

    final promotionGetPromotionsQuery = PromotionGetPromotionsQuery(
      start: start,
      limit: limit,
      categoryIds: categoryIds,
      storeIds: storeIds,
      retailerTokens: ConfigReader.clientType != ClientType.RETAILER
          ? retailerTokens
          : [ConfigReader.clientApiToken],
      companyTokens: ConfigReader.clientType != ClientType.COMPANY
          ? companyTokens
          : [ConfigReader.clientApiToken],
      textSearch: textSearch,
      zipcode: zipcode,
      latitude: latitude,
      longitude: longitude,
      radius: radius,
      orderBy: orderBy,
      publisherToken: publisherToken,
    );

    try {
      final response = await dio.get(
        PublisherApiHelper.getPromotionsUrl(),
        queryParameters: promotionGetPromotionsQuery.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      print(e.toString());
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getPromotionDetail({
    String? promotionToken,
    String? publisherToken,
  }) async {
    dio.options.headers = PublisherApiHelper.buildHeaders();

    final promotionGetPromotionDetailQuery = PromotionGetPromotionDetailQuery(
      publisherToken: publisherToken,
    );

    try {
      final response = await dio.get(
        PublisherApiHelper.getPromotionDetailUrl(promotionToken!),
        queryParameters: promotionGetPromotionDetailQuery.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      print(e.toString());
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getCategories({
    String? publisherToken,
  }) async {
    dio.options.headers = PublisherApiHelper.buildHeaders();

    final promotionGetFilterQuery = PromotionGetFilterQuery(
      publisherToken: publisherToken,
      lang: ConfigReader.languageCode,
    );

    try {
      final response = await dio.get(
        PublisherApiHelper.getCategoriesUrl(),
        queryParameters: promotionGetFilterQuery.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      print(e.toString());
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getCompanies({
    String? publisherToken,
    int? start,
    int? limit,
  }) async {
    dio.options.headers = PublisherApiHelper.buildHeaders();

    final promotionGetFilterQuery = PromotionGetFilterQuery(
      publisherToken: publisherToken,
      lang: ConfigReader.languageCode,
      start: start,
      limit: limit,
    );

    try {
      final response = await dio.get(
        PublisherApiHelper.getCompaniesUrl(),
        queryParameters: promotionGetFilterQuery.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      print(e.toString());
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getRetailers({
    String? publisherToken,
    int? start,
    int? limit,
  }) async {
    dio.options.headers = PublisherApiHelper.buildHeaders();

    final promotionGetFilterQuery = PromotionGetFilterQuery(
      publisherToken: publisherToken,
      lang: ConfigReader.languageCode,
      start: start,
      limit: limit,
    );

    try {
      final response = await dio.get(
        PublisherApiHelper.getRetailersUrl(),
        queryParameters: promotionGetFilterQuery.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      print(e.toString());
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }
}
