import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/coupons_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/redemption_get_redemption_detail_query.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/redemption_get_user_redemption_history_query.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/redemption_get_user_redemption_summary_query.dart';

import 'package:dcoupon_sdk_flutter_package/src/data/helpers/users_api_helper.dart';

import 'package:dio/dio.dart';

abstract class RedemptionRemoteDataSource {
  Future<Either<Failure, dynamic>> getUserRedemptionSummary({
    required String groupBy,
    String? startDate,
    String? endDate,
    int? gmtTimeZoneOffset = 0,
  });

  Future<Either<Failure, dynamic>> getUserRedemptionHistory({
    int? limit,
    int? offset,
    String? startDate,
    String? endDate,
    int? gmtTimeZoneOffset = 0,
  });

  Future<Either<Failure, dynamic>> getRedemptionDetail({
    required String? redemptionId,
    int? gmtTimeZoneOffset = 0,
  });

  Future<Either<Failure, dynamic>> getRedemptionCode();
}

class RedemptionRemoteDataSourceImpl implements RedemptionRemoteDataSource {
  Dio dio = Dio()..interceptors.addAll(ConfigReader.dioInterceptors);

  @override
  Future<Either<Failure, dynamic>> getUserRedemptionSummary({
    required String groupBy,
    String? startDate,
    String? endDate,
    int? gmtTimeZoneOffset = 0,
  }) async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    final queryParameters = RedemptionGetUserRedemptionSummaryQuery(
      groupBy: groupBy,
      startDate: startDate,
      endDate: endDate,
      gmtTimeZoneOffset: gmtTimeZoneOffset,
    );

    try {
      final response = await dio.get(
        UsersApiHelper.getUserRedemptionSummaryUrl(),
        queryParameters: queryParameters.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserRedemptionHistory({
    int? limit,
    int? offset,
    String? startDate,
    String? endDate,
    int? gmtTimeZoneOffset = 0,
  }) async {
    // dio.interceptors.add(LoggingInterceptor());
    dio.options.headers = UsersApiHelper.buildHeaders();

    final queryParameters = RedemptionGetUserRedemptionHistoryQuery(
      limit: limit,
      offset: offset,
      startDate: startDate,
      endDate: endDate,
      gmtTimeZoneOffset: gmtTimeZoneOffset,
      lang: ConfigReader.languageCode,
      country: ConfigReader.clientCountry,
    );

    try {
      final response = await dio.get(
        UsersApiHelper.getUserRedemptionHistoryUrl(),
        queryParameters: queryParameters.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getRedemptionDetail({
    required String? redemptionId,
    int? gmtTimeZoneOffset = 0,
  }) async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    final queryParameters = RedemptionGetRedemptionDetailQuery(
      gmtTimeZoneOffset: gmtTimeZoneOffset,
    );

    try {
      final response = await dio.get(
        UsersApiHelper.getRedemptionDetail(redemptionId!),
        queryParameters: queryParameters.toJson(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getRedemptionCode() async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    try {
      final response = await dio.get(
        CouponsApiHelper.getRedemptionTokenUrl(),
      );
      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }
}
