import 'dart:convert';
import 'dart:io';

import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/third_party_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/helpers/users_api_helper.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_accept_enrollment_form_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_accept_enrollment_form_response.dart';

import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_request.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_update_user_data_request.dart';
import 'package:dio/dio.dart';

abstract class UserRemoteDataSource {
  Future<Either<Failure, UserLoginResponse>> login({
    required String externalId,
    required String? email,
    String? referralCode,
  });

  Future<Either<Failure, UserAcceptEnrollmentFormResponse>>
      acceptEnrollmentForm({
    required String? jwt,
    required List<String>? legalDocumentsAccepted,
  });

  Future<Either<Failure, dynamic>> getUserData();

  Future<Either<Failure, dynamic>> updateUserData({
    String? alias,
    String? gender,
    String? birthdate,
  });
}

class UserRemoteDataSourceImpl implements UserRemoteDataSource {
  Dio dio = Dio()..interceptors.addAll(ConfigReader.dioInterceptors);

  @override
  Future<Either<Failure, UserLoginResponse>> login({
    required String externalId,
    required String? email,
    String? referralCode,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = ThirdPartyAPIHelper.buildHeaders();

    final userLoginRequest = UserLoginRequest(
      externalId: externalId,
      email: email,
      referralCode: referralCode,
    );

    try {
      final response = await dio.post(
        ThirdPartyAPIHelper.loginUrl(),
        data: userLoginRequest.toJson(),
      );

      UserLoginResponse userLoginResponse;
      if (response.statusCode == HttpStatus.accepted) {
        userLoginResponse =
            UserLoginEnrollmentNeededResponse.fromResponse(response);
      } else {
        userLoginResponse = UserLoginSuccessResponse.fromResponse(response);
      }
      return Right(userLoginResponse);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, UserAcceptEnrollmentFormResponse>>
      acceptEnrollmentForm({
    required String? jwt,
    required List<String>? legalDocumentsAccepted,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = ThirdPartyAPIHelper.buildHeaders();

    final userAcceptEnrollmentFormRequest = UserAcceptEnrollmentFormRequest(
      jwt: jwt,
      legalDocumentsAccepted: legalDocumentsAccepted,
    );

    try {
      final data = jsonEncode(userAcceptEnrollmentFormRequest.toJson());
      final response = await dio.post(
        ThirdPartyAPIHelper.acceptEnrollmentFormUrl(),
        data: data,
      );
      final userAcceptEnrollmentFormResponse =
          UserAcceptEnrollmentFormResponse.fromResponse(response);
      return Right(userAcceptEnrollmentFormResponse);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserData() async {
    dio.options.headers = UsersApiHelper.buildHeaders();

    try {
      final response = await dio.get(
        UsersApiHelper.getUserDataUrl(),
      );

      final data = response.data as Map<String, dynamic>?;

      return Right(data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }

  @override
  Future<Either<Failure, dynamic>> updateUserData({
    String? alias,
    String? gender,
    String? birthdate,
  }) async {
    dio.options.contentType = Headers.jsonContentType;

    dio.options.headers = UsersApiHelper.buildHeaders();

    final userUpdateUserDataRequest = UserUpdateUserDataRequest(
      alias: alias,
      gender: gender,
      birthdate: birthdate,
    );

    try {
      final response = await dio.put(
        UsersApiHelper.updateUserDataUrl(),
        data: userUpdateUserDataRequest.toJson(),
      );

      return Right(response.data);
    } on DioError catch (e) {
      if (e.type == DioErrorType.response) {
        return Left(ServerFailure(httpStatus: e.response!.statusCode));
      } else {
        throw ServerException(e.message);
      }
    }
  }
}
